#ifndef _Inline_namespace_h
#define _Inline_namespace_h 

#include <chrono>
#include <ctime> // 'std::size_t'...
#include <set>
#include <string>
#include <utility> // 'std::move'...

namespace New_York_City_Penitentiary {
    inline namespace DAOs {

        class Criminal_case {
        };

        class Motive {
        };
    }
    namespace Nominal {

        class Prisoner {
            const std::string _prison_file_number;
            std::string _given_name;
            std::string _surname;
            std::tm _date_of_birth;
            std::string _place_of_birth;
            std::tm _date_of_incarceration;
            const Criminal_case * _incarceration_main;
        public:
            explicit Prisoner(const std::string&);
            bool operator<(const Prisoner&) const; // For 'std::set'
            std::string get_prison_file_number() const;
            std::string get_given_name() const;
            void set_given_name(const std::string&);
            const std::string& get_surname() const;
            void set_surname(const std::string&);
            std::tm get_date_of_birth() const;
            void set_date_of_birth(const std::tm&);
            std::string get_place_of_birth() const;
            void set_place_of_birth(const std::string&);
            std::tm get_date_of_incarceration() const;
            void set_date_of_incarceration(const std::tm&);
            const Criminal_case* get_incarceration_main() const;
            void set_incarceration_main(const Criminal_case *);
            std::string toString();
        };

        class Prisoner_hash { // For 'std::unordered_set'
        public:
            std::size_t operator()(Prisoner const&);
        };
    }
    inline namespace Ver_2 {

        class Prisoner {
            const std::string _prison_file_number;
            std::string _given_name;
            std::string _surname;
            std::chrono::system_clock::time_point _date_of_birth; // 'chrono'
            std::string _place_of_birth;
            std::chrono::system_clock::time_point _date_of_incarceration; // 'chrono'
            const Criminal_case * _incarceration_main;
        public:
            explicit Prisoner(std::string&&);
            std::string get_prison_file_number() const;
            std::string get_given_name() const;
            void set_given_name(const std::string&);
            const std::string& get_surname() const;
            void set_surname(const std::string&);
            std::tm get_date_of_birth() const;
            void set_date_of_birth(std::tm&&);
            std::string get_place_of_birth() const;
            void set_place_of_birth(const std::string&);
            std::tm get_date_of_incarceration() const;
            void set_date_of_incarceration(std::tm&&);
            const Criminal_case* get_incarceration_main() const;
            void set_incarceration_main(const Criminal_case *);
            std::string toString();

            std::set<const Criminal_case *> _offense;
            std::set<const Criminal_case *> get_offense() const;
            void add_offense(const Criminal_case *);

            //            const Motive * _incarceration_motive;
            //            const Motive* get_incarceration_motive() const;
            //            void set_incarceration_motive(const Motive *);

        };
    }
    // Literal suffix (re)-definition cannot be a member function: 
    Prisoner operator ""_Prisoner(const char[], size_t); // '_' is required not to conflict with hypothetical (future) use of 'Prisoner' as suffix...
}

#endif          
