#include <ctime> 
#include <string>

#include "Inline_namespace.h"

New_York_City_Penitentiary::Nominal::Prisoner::Prisoner(const std::string& prison_file_number) : _prison_file_number(prison_file_number) {
}

bool New_York_City_Penitentiary::Nominal::Prisoner::operator<(const Prisoner& p) const {
    // Note that '_prison_file_number' is accessible even though 'private:'...
    return this->_prison_file_number < p._prison_file_number;
}

std::string New_York_City_Penitentiary::Nominal::Prisoner::get_prison_file_number() const {
    return _prison_file_number;
}

std::string New_York_City_Penitentiary::Nominal::Prisoner::get_given_name() const {
    return _given_name;
}

void New_York_City_Penitentiary::Nominal::Prisoner::set_given_name(const std::string& given_name) {
    _given_name = given_name;
}

const std::string& New_York_City_Penitentiary::Nominal::Prisoner::get_surname() const {
    return _surname;
}

void New_York_City_Penitentiary::Nominal::Prisoner::set_surname(const std::string& surname) {
    _surname = surname;
}

std::tm New_York_City_Penitentiary::Nominal::Prisoner::get_date_of_birth() const {
    return _date_of_birth;
}

void New_York_City_Penitentiary::Nominal::Prisoner::set_date_of_birth(const std::tm& date_of_birth) {
    _date_of_birth = date_of_birth;
}

std::string New_York_City_Penitentiary::Nominal::Prisoner::get_place_of_birth() const {
    return _place_of_birth;
}

void New_York_City_Penitentiary::Nominal::Prisoner::set_place_of_birth(const std::string& place_of_birth) {
    _place_of_birth = place_of_birth;
}

std::tm New_York_City_Penitentiary::Nominal::Prisoner::get_date_of_incarceration() const {
    return _date_of_incarceration;
}

void New_York_City_Penitentiary::Nominal::Prisoner::set_date_of_incarceration(const std::tm& date_of_incarceration) {
    _date_of_incarceration = date_of_incarceration;
}

const New_York_City_Penitentiary::Criminal_case* New_York_City_Penitentiary::Nominal::Prisoner::get_incarceration_main() const {
    return _incarceration_main;
}

void New_York_City_Penitentiary::Nominal::Prisoner::set_incarceration_main(const Criminal_case * incarceration_main) {
    _incarceration_main = incarceration_main;
}

std::string New_York_City_Penitentiary::Nominal::Prisoner::toString() {
    return _prison_file_number + '-' + _given_name + '-' + _surname;
}

std::size_t New_York_City_Penitentiary::Nominal::Prisoner_hash::operator()(New_York_City_Penitentiary::Nominal::Prisoner const& p) const noexcept {
    return std::hash<std::string>{}(p._prison_file_number);
}

// Ver_2 is not mentioned because 'default':

New_York_City_Penitentiary::Prisoner::Prisoner(std::string&& prison_file_number) : _prison_file_number(prison_file_number) {
}

void New_York_City_Penitentiary::Prisoner::set_date_of_incarceration(std::tm&& date_of_incarceration) {
    _date_of_incarceration = std::chrono::system_clock::from_time_t(std::mktime(&date_of_incarceration));
}

void New_York_City_Penitentiary::Prisoner::set_incarceration_main(const Criminal_case * criminal_case) {
    _offense.insert(criminal_case);
    _incarceration_main = criminal_case;
}

std::set<const New_York_City_Penitentiary::Criminal_case *> New_York_City_Penitentiary::Prisoner::get_offense() const {
    return _offense;
}

void New_York_City_Penitentiary::Prisoner::add_offense(const New_York_City_Penitentiary::Criminal_case * criminal_case) {
    _offense.insert(criminal_case);
}

New_York_City_Penitentiary::Prisoner New_York_City_Penitentiary::operator ""_Prisoner(const char prison_file_number[], size_t size) {
    std::string _prison_file_number(prison_file_number);
    return New_York_City_Penitentiary::Prisoner(std::move(_prison_file_number));
}

int main(int argc, char** argv) {
    using namespace New_York_City_Penitentiary;
    using namespace std::literals::string_literals; // Operator""s is made visible...

    Criminal_case criminal_case; // 'DAOs::Criminal_case' is not required...

    Nominal::Prisoner p("1630125388055-29"s); // Explicit "nominal" version...
    p.set_date_of_incarceration(std::tm{.tm_mday = 13, .tm_mon = 10, .tm_year = 119}); // 13/11/2019
    p.set_incarceration_main(&criminal_case);

    Prisoner p_("1630125388055-29"s); // Last version, i.e., 'Ver_2'...
    p.set_date_of_incarceration(std::tm{.tm_mday = 13, .tm_mon = 10, .tm_year = 119}); // 13/11/2019
    p_.set_incarceration_main(&criminal_case); // Do much: '_offense.insert(criminal_case);'

    //    Prisoner fb = "1630125388055-29"s; // Compilation error: 'explicit' constructor precludes implicit conversion...
    Prisoner fb = "1630125388055-29"_Prisoner;

    return 0;
}

